from __future__ import unicode_literals, print_function
from django.contrib.auth import get_user_model
from django.utils.timezone import now, timedelta
from rest_framework.test import APITestCase as BaseAPITestCase
from oauth2_provider.models import get_application_model, AccessToken


class TestCase(BaseAPITestCase):
    """
    This is the base test case providing common features for all tests
    """

    def setUp(self):
        """
        Create a superuser and standard user.
        Create an Oauth2 app for superuser.
        """
        self.superuser = get_user_model().objects.create_superuser('admin', password='admin', email='admin@example.com')
        self.user = get_user_model().objects.create_user('test', password='test', email='test@example.com')

        self.app = get_application_model().objects.create(
            name='app',
            client_type=get_application_model().CLIENT_CONFIDENTIAL,
            authorization_grant_type=get_application_model().GRANT_CLIENT_CREDENTIALS,
            user=self.superuser
        )

        self.access_token = AccessToken.objects.create(user=self.superuser,
                                                       token='token_monster',
                                                       application=self.app,
                                                       expires=now() + timedelta(seconds=300))

        self.auth_valid = self._gen_authorization_header(self.access_token.token)
        self.auth_invalid = self._gen_authorization_header("fake-token")

    def _gen_authorization_header(self, token):
        return "Bearer {0}".format(token)

    def tearDown(self):
        """
        Clean up when tests end
        """
        self.superuser.delete()
        self.user.delete()
        self.app.delete()
        self.access_token.delete()

    def get_categories_as_delim_str(self, obj):
        return ",".join([p.title for p in obj.categories.all()])



from __future__ import unicode_literals, print_function
from django.core.urlresolvers import reverse
from rest_framework import status
from mezzanine.blog.models import BlogCategory
from tests.utils import TestCase


class TestCategoryViewSet(TestCase):
    """
    Test the API resources for categories
    """

    def setUp(self):
        """
        Setup the tests
        Create a category for API retrieval testing
        """
        super(TestCategoryViewSet, self).setUp()
        self.category = BlogCategory.objects.create(title='Fitness')

    def tearDown(self):
        """
        Clean up after the tests
        """
        self.category.delete()

    def test_list(self):
        """
        Test API list
        """
        url = reverse('blogcategory-list')
        response = self.client.get(url, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 1)

    def test_retrieve(self):
        """
        Test API retrieve
        """
        url = '/api/categories/{}'.format(self.category.pk)
        response = self.client.get(url, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['title'], self.category.title)

    def test_create_as_superuser_token(self):
        """
        Test API POST CREATE whilst authenticated via OAuth2 as a superuser
        """
        post_data = {'title': 'my new category 1'}
        url = '/api/categories'
        response = self.client.post(url, post_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(BlogCategory.objects.get(pk=response.data['id']).title, post_data['title'])

    def test_create_as_superuser(self):
        """
        Test API POST CREATE whilst authenticated as a superuser
        """
        post_data = {'title': 'my new category 2'}
        url = '/api/categories'
        self.client.force_authenticate(user=self.superuser)
        response = self.client.post(url, post_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(BlogCategory.objects.get(pk=response.data['id']).title, post_data['title'])

    def test_create_as_user(self):
        """
        Test API POST CREATE whilst authenticated as a standard user
        """
        post_data = {'title': 'my category'}
        url = '/api/categories'
        self.client.force_authenticate(user=self.user)
        response = self.client.post(url, post_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_create_as_guest(self):
        """
        Test API POST CREATE whilst unauthenticated as a guest
        """
        post_data = {'title': 'my category'}
        url = '/api/categories'
        self.client.force_authenticate(user=None)
        response = self.client.post(url, post_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_update_as_superuser_token(self):
        """
        Test API PUT UPDATE whilst authenticated via OAuth2 as a superuser
        """
        put_data = {'title': 'my updated category'}
        url = '/api/categories/{}'.format(self.category.pk)
        response = self.client.put(url, put_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(BlogCategory.objects.get(pk=self.category.pk).title, put_data['title'])

    def test_update_as_user(self):
        """
        Test API PUT UPDATE whilst authenticated as a standard user
        """
        put_data = {'title': 'my updated category'}
        url = '/api/categories/{}'.format(self.category.pk)
        self.client.force_authenticate(user=self.user)
        response = self.client.put(url, put_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_update_as_guest(self):
        """
        Test API PUT UPDATE whilst unauthenticated as a guest
        """
        put_data = {'title': 'my updated category'}
        url = '/api/categories/{}'.format(self.category.pk)
        self.client.force_authenticate(user=None)
        response = self.client.put(url, put_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)



from __future__ import unicode_literals, print_function
from django.core.urlresolvers import reverse
from rest_framework import status
from mezzanine.blog.models import BlogPost as Post
from tests.utils import TestCase


class TestPostViewSet(TestCase):
    """
    Test the API resources for blog posts (read and write)
    """

    def setUp(self):
        """
        Setup the tests
        Create some published and draft blog posts for API retrieval testing
        """
        super(TestPostViewSet, self).setUp()

        # Note for using status:
        # from mezzanine.core.models import CONTENT_STATUS_PUBLISHED
        # status=CONTENT_STATUS_PUBLISHED

        self.post_draft = Post.objects.create(
            title="Draft Post Title",
            content="Draft Content",
            user=self.user)

        self.post_published = Post.objects.create(
            title="Published Post Title",
            content="Published Content",
            publish_date='2016-01-01T00:00',
            user=self.user)

    def tearDown(self):
        """
        Clean up after the tests
        """
        super(TestPostViewSet, self).tearDown()
        self.post_draft.delete()
        self.post_published.delete()

    def test_list_published_posts(self):
        """
        Test API list all published blog posts
        """
        url = reverse('blogpost-list')
        response = self.client.get(url, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response['Content-type'], 'application/json')
        self.assertEqual(response.data['count'], 1)
        self.assertEqual(response.data['results'][0]['title'], self.post_published.title)

    def test_retrieve_published_post(self):
        """
        Test API retrieve the published blog post that we created earlier
        """
        url = '/api/posts/{}'.format(self.post_published.pk)
        response = self.client.get(url, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['title'], self.post_published.title)

    def test_retrieve_draft_post(self):
        """
        Test that retrieving a draft post fails since the API only allows read access to published posts
        """
        url = '/api/posts/{}'.format(self.post_draft.pk)
        response = self.client.get(url, format='json')

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_create_as_superuser_token(self):
        """
        Test API POST CREATE whilst authenticated via OAuth2 as a superuser
        """
        # Note: we do not directly provide user here, as API should automatically get and
        # authenticate current user as the author
        post_data = {'title': 'title1', 'content': 'content1', 'publish_date': '2016-01-01T00:00',
                     'categories': 'Machine Learning,Statistics'}
        url = '/api/posts'
        response = self.client.post(url, post_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Post.objects.get(pk=response.data['id']).user, self.superuser)
        self.assertEqual(Post.objects.get(pk=response.data['id']).title, post_data['title'])
        self.assertEqual(Post.objects.get(pk=response.data['id']).content, post_data['content'])
        self.assertEqual(self.get_categories_as_delim_str(Post.objects.get(pk=response.data['id'])),
                         post_data['categories'])

    def test_create_as_superuser(self):
        """
        Test API POST CREATE whilst authenticated as a superuser
        """
        post_data = {'title': 'title2', 'content': 'content2', 'publish_date': '2016-01-01T00:00',
                     'categories': 'Machine Learning'}
        url = '/api/posts'
        self.client.force_authenticate(user=self.superuser)
        response = self.client.post(url, post_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Post.objects.get(pk=response.data['id']).user, self.superuser)
        self.assertEqual(Post.objects.get(pk=response.data['id']).title, post_data['title'])
        self.assertEqual(Post.objects.get(pk=response.data['id']).content, post_data['content'])
        self.assertEqual(self.get_categories_as_delim_str(Post.objects.get(pk=response.data['id'])),
                         post_data['categories'])

    def test_create_as_user(self):
        """
        Test API POST CREATE whilst authenticated as a standard user
        """
        post_data = {'title': 'a', 'content': 'b', 'publish_date': '2016-01-01T00:00'}
        url = '/api/posts'
        self.client.force_authenticate(user=self.user)
        response = self.client.post(url, post_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_create_as_guest(self):
        """
        Test API POST CREATE whilst unauthenticated as a guest
        """
        post_data = {'title': 'a', 'content': 'b', 'publish_date': '2016-01-01T00:00'}
        url = '/api/posts'
        self.client.force_authenticate(user=None)
        response = self.client.post(url, post_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_update_as_superuser_token(self):
        """
        Test API PUT UPDATE whilst authenticated via OAuth2 as a superuser
        """
        put_data = {'title': 'a', 'content': 'b', 'categories': 'cat1,cat2'}
        url = '/api/posts/{}'.format(self.post_published.pk)
        response = self.client.put(url, put_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).title, put_data['title'])
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).content, put_data['content'])
        self.assertEqual(self.get_categories_as_delim_str(Post.objects.get(pk=self.post_published.pk)),
                         put_data['categories'])

    def test_update_as_user(self):
        """
        Test API PUT UPDATE whilst authenticated as a standard user
        """
        put_data = {'title': 'a', 'content': 'b'}
        url = '/api/posts/{}'.format(self.post_published.pk)
        self.client.force_authenticate(user=self.user)
        response = self.client.put(url, put_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_update_as_guest(self):
        """
        Test API PUT UPDATE whilst unauthenticated as a guest
        """
        put_data = {'title': 'a', 'content': 'b'}
        url = '/api/posts/{}'.format(self.post_published.pk)
        self.client.force_authenticate(user=None)
        response = self.client.put(url, put_data, format='json')

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_update_categories_unchanged(self):
        """
        Test API PUT UPDATE title and test categories remain unchanged
        whilst authenticated via OAuth2 as a superuser
        """
        original_content = Post.objects.get(pk=self.post_published.pk).content
        original_categories = self.get_categories_as_delim_str(Post.objects.get(pk=self.post_published.pk))

        put_data = {'title': 'updated title'}
        url = '/api/posts/{}'.format(self.post_published.pk)
        response = self.client.put(url, put_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).title, put_data['title'])
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).content, original_content)
        self.assertEqual(self.get_categories_as_delim_str(Post.objects.get(pk=self.post_published.pk)),
                         original_categories)

    def test_update_categories_disassociate_one(self):
        """
        Test API PUT UPDATE disassociate category 'cat2'
        whilst authenticated via OAuth2 as a superuser
        """
        original_title = Post.objects.get(pk=self.post_published.pk).title
        original_content = Post.objects.get(pk=self.post_published.pk).content

        put_data = {'categories': 'cat1'}
        url = '/api/posts/{}'.format(self.post_published.pk)
        response = self.client.put(url, put_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)
        print(response.data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).title, original_title)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).content, original_content)
        self.assertEqual(self.get_categories_as_delim_str(Post.objects.get(pk=self.post_published.pk)),
                         put_data['categories'])

    def test_update_categories_associate_three(self):
        """
        Test API PUT UPDATE associate three new categories
        whilst authenticated via OAuth2 as a superuser
        """
        original_title = Post.objects.get(pk=self.post_published.pk).title
        original_content = Post.objects.get(pk=self.post_published.pk).content

        put_data = {'categories': 'cat1,cat2,cat3,cat4'}
        url = '/api/posts/{}'.format(self.post_published.pk)
        response = self.client.put(url, put_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)
        print(response.data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).title, original_title)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).content, original_content)
        self.assertEqual(self.get_categories_as_delim_str(Post.objects.get(pk=self.post_published.pk)),
                         put_data['categories'])

    def test_update_categories_disassociate_all(self):
        """
        Test API PUT UPDATE disassociate all categories
        whilst authenticated via OAuth2 as a superuser
        """
        original_title = Post.objects.get(pk=self.post_published.pk).title
        original_content = Post.objects.get(pk=self.post_published.pk).content

        put_data = {'categories': ''}
        url = '/api/posts/{}'.format(self.post_published.pk)
        response = self.client.put(url, put_data, format='json', HTTP_AUTHORIZATION=self.auth_valid)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).title, original_title)
        self.assertEqual(Post.objects.get(pk=self.post_published.pk).content, original_content)
        self.assertEqual(self.get_categories_as_delim_str(Post.objects.get(pk=self.post_published.pk)),
                         put_data['categories'])



REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'oauth2_provider.ext.rest_framework.OAuth2Authentication',
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
    ),
    'DEFAULT_FILTER_BACKENDS': ('rest_framework.filters.DjangoFilterBackend',),
    'TEST_REQUEST_DEFAULT_FORMAT': 'json',
}

OAUTH2_PROVIDER = {
    # List of available scopes
    'SCOPES': {'read': 'Read scope', 'write': 'Write scope', 'groups': 'Access to your groups'}
}

# API login URL for oauth2_provider (based on default routing in urls.py)
LOGIN_URL = "/api/auth/login/"

SWAGGER_SETTINGS = {
    'exclude_namespaces': [],
    'api_version': '',
    'api_path': '/',
    'api_key': '',  # Your OAuth2 Access Token
    'token_type': 'Bearer',
    'is_authenticated': False,
    'is_superuser': False,
    'permission_denied_handler': None,
    'info': {
        'title': 'API Resource Documentation',
        'description': 'The RESTful web API exposes Mezzanine data using JSON serialization and OAuth2 protection. '
                       'This interactive document will guide you through the relevant API endpoints, data structures, '
                       'and query parameters for filtering, searching and pagination. Otherwise, for further '
                       'information and examples, consult the general '
                       '<a href="http://gcushen.github.io/mezzanine-api" target="_blank">Mezzanine API Documentation'
                       '</a>.',
    },
    'doc_expansion': 'none',
}



# Bare ``settings.py`` for running tests for mezzanine_api
from __future__ import absolute_import, unicode_literals
import os

from django import VERSION as DJANGO_VERSION
from django.utils.translation import ugettext_lazy as _


DEBUG = True

ROOT_URLCONF = 'tests.urls'

SECRET_KEY = '12345'

USE_SOUTH = True

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)
MANAGERS = ADMINS

USE_MODELTRANSLATION = False

ALLOWED_HOSTS = ['testserver']

TIME_ZONE = 'Europe/London'

USE_TZ = True

LANGUAGE_CODE = "en"

# Supported languages
LANGUAGES = (
    ('en', _('English')),
)

SESSION_EXPIRE_AT_BROWSER_CLOSE = True

SITE_ID = 1

USE_I18N = False

AUTHENTICATION_BACKENDS = ("mezzanine.core.auth_backends.MezzanineBackend",)

FILE_UPLOAD_PERMISSIONS = 0o644

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'dev.sqlite',
    }
}

PROJECT_APP_PATH = os.path.dirname(os.path.abspath(__file__))
PROJECT_APP = os.path.basename(PROJECT_APP_PATH)
PROJECT_ROOT = BASE_DIR = os.path.dirname(PROJECT_APP_PATH)

CACHE_MIDDLEWARE_KEY_PREFIX = PROJECT_APP

STATIC_URL = "/static/"

STATIC_ROOT = os.path.join(PROJECT_ROOT, STATIC_URL.strip("/"))

MEDIA_URL = STATIC_URL + "media/"

MEDIA_ROOT = os.path.join(PROJECT_ROOT, *MEDIA_URL.strip("/").split("/"))

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(PROJECT_ROOT, "templates")
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.static",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.tz",
                "mezzanine.conf.context_processors.settings",
                "mezzanine.pages.context_processors.page",
            ],
            "builtins": [
                "mezzanine.template.loader_tags",
            ],
        },
    },
]

if DJANGO_VERSION < (1, 9):
    del TEMPLATES[0]["OPTIONS"]["builtins"]

INSTALLED_APPS = (
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.redirects",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.sitemaps",
    "django.contrib.staticfiles",
    "mezzanine.boot",
    "mezzanine.conf",
    "mezzanine.core",
    "mezzanine.generic",
    "mezzanine.pages",
    "mezzanine.blog",
    "mezzanine.forms",
    "mezzanine.galleries",
    "mezzanine.twitter",
    'mezzanine_api',
    'rest_framework',
    'rest_framework_swagger',
    'oauth2_provider',
)

MIDDLEWARE_CLASSES = (
    "mezzanine.core.middleware.UpdateCacheMiddleware",

    'django.contrib.sessions.middleware.SessionMiddleware',
    # Uncomment if using internationalisation or localisation
    # 'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',

    "mezzanine.core.request.CurrentRequestMiddleware",
    "mezzanine.core.middleware.RedirectFallbackMiddleware",
    "mezzanine.core.middleware.TemplateForDeviceMiddleware",
    "mezzanine.core.middleware.TemplateForHostMiddleware",
    "mezzanine.core.middleware.AdminLoginInterfaceSelectorMiddleware",
    "mezzanine.core.middleware.SitePermissionMiddleware",
    # Uncomment the following if using any of the SSL settings:
    # "mezzanine.core.middleware.SSLRedirectMiddleware",
    "mezzanine.pages.middleware.PageMiddleware",
    "mezzanine.core.middleware.FetchFromCacheMiddleware",
)

# Store these package names here as they may change in the future since
# at the moment we are using custom forks of them.
PACKAGE_NAME_FILEBROWSER = "filebrowser_safe"
PACKAGE_NAME_GRAPPELLI = "grappelli_safe"

#########################
# OPTIONAL APPLICATIONS #
#########################

# These will be added to ``INSTALLED_APPS``, only if available.
OPTIONAL_APPS = (
    "debug_toolbar",
    "django_extensions",
    "compressor",
    PACKAGE_NAME_FILEBROWSER,
    PACKAGE_NAME_GRAPPELLI,
)

##################
# LOCAL SETTINGS #
##################
# Included above


####################
# DYNAMIC SETTINGS #
####################
try:
    from mezzanine.utils.conf import set_dynamic_settings
except ImportError:
    pass
else:
    set_dynamic_settings(globals())


#####################
# REST API SETTINGS #
#####################
try:
    from mezzanine_api.settings import *  # NOQA
except ImportError as e:
    pass



from rest_framework import viewsets, filters, permissions, mixins
from rest_framework.pagination import PageNumberPagination
import django_filters

from .serializers import UserSerializer, CategorySerializer, PostInputSerializer, PostOutputSerializer
from .serializers import PageSerializer, SiteSerializer
from .permissions import IsAdminOrReadOnly
from .mixins import PutUpdateModelMixin

from django.contrib.auth.models import User
from mezzanine.blog.models import BlogPost as Post, BlogCategory
from mezzanine.pages.models import Page
from django.contrib.sites.models import Site

# from oauth2_provider.ext.rest_framework import TokenHasReadWriteScope, TokenHasScope


class CustomPagination(PageNumberPagination):
    """
    Default pagination class.
    Let large result sets be split into individual pages of data.
    """
    page_size = 20
    page_size_query_param = 'page_size'
    max_page_size = 100


class PostPagination(CustomPagination):
    """
    Pagination for Blog Posts
    """
    page_size = 5


class ListViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    """
    A viewset that provides only `list` actions.

    To use it, override the class and set the `.queryset` and `.serializer_class` attributes.
    """
    pass


class SiteViewSet(ListViewSet):
    """
    For retrieving site title, tagline and domain.
    """
    queryset = Site.objects.all()
    serializer_class = SiteSerializer


class UserFilter(django_filters.FilterSet):
    """
    A class for filtering users.
    """
    username = django_filters.CharFilter(name="username", lookup_type='istartswith')

    class Meta:
        model = User
        fields = ['username']


class UserViewSet(viewsets.ReadOnlyModelViewSet):
    """
    For listing or retrieving users.
    ---
    list:
        parameters:
            - name: username
              type: string
              description: Filter usernames starting with query
              paramType: query
            - name: page
              type: integer
              description: Page number
              paramType: query
    """
    queryset = User.objects.all()
    filter_class = UserFilter
    filter_backends = (filters.DjangoFilterBackend,)
    serializer_class = UserSerializer
    pagination_class = CustomPagination
    permission_classes = (permissions.IsAdminUser,)


class PageViewSet(viewsets.ReadOnlyModelViewSet):
    """
    For listing or retrieving pages.
    ---
    list:
        parameters:
            - name: page
              type: integer
              description: Page number
              paramType: query
    """
    queryset = Page.objects.published()
    serializer_class = PageSerializer
    pagination_class = CustomPagination

    def get_queryset(self):
        queryset = self.queryset
        user = self.request.user

        if not user.is_authenticated():
            queryset = queryset.filter(login_required=False)

        return queryset


class CategoryViewSet(mixins.CreateModelMixin,
                      mixins.RetrieveModelMixin,
                      PutUpdateModelMixin,
                      mixins.ListModelMixin,
                      viewsets.GenericViewSet):
    """
    For listing, retrieving, creating or updating blog categories.
    ---
    list:
        parameters:
            - name: search
              type: string
              description: Search for category names that match the query
              paramType: query
            - name: page
              type: integer
              description: Page number
              paramType: query
    """
    queryset = BlogCategory.objects.all()
    filter_backends = (filters.SearchFilter,)
    search_fields = ('title',)
    serializer_class = CategorySerializer
    pagination_class = CustomPagination
    permission_classes = [IsAdminOrReadOnly]


class PostFilter(django_filters.FilterSet):
    """
    A class for filtering blog posts.
    """
    category_id = django_filters.NumberFilter(name="categories__id")
    category_name = django_filters.CharFilter(name="categories__title", lookup_type='contains')
    category_slug = django_filters.CharFilter(name="categories__slug", lookup_type='exact')
    tag = django_filters.CharFilter(name='keywords_string', lookup_type='contains')
    author_id = django_filters.NumberFilter(name="user__id")
    author_name = django_filters.CharFilter(name="user__username", lookup_type='istartswith')
    date_min = django_filters.DateFilter(name='publish_date', lookup_type='gte')
    date_max = django_filters.DateFilter(name='publish_date', lookup_type='lte')

    class Meta:
        model = Post
        fields = ['category_id', 'category_name', 'tag', 'author_id', 'author_name', 'date_min', 'date_max']


class PostViewSet(mixins.CreateModelMixin,
                  mixins.RetrieveModelMixin,
                  PutUpdateModelMixin,
                  mixins.ListModelMixin,
                  viewsets.GenericViewSet):
    """
    For listing, retrieving, creating or updating blog posts.
    ---
    list:
        parameters:
            - name: category_id
              type: integer
              description: Filter posts by category ID
              paramType: query
            - name: category_name
              type: string
              description: Filter posts by category name
              paramType: query
            - name: category_slug
              type: string
              description: Filter posts by category slug
              paramType: query
            - name: tag
              type: string
              description: Filter posts by tag name
              paramType: query
            - name: author_id
              type: integer
              description: Filter posts by author ID
              paramType: query
            - name: author_name
              type: string
              description: Filter posts by author's username
              paramType: query
            - name: date_min
              type: datetime
              description: Filter posts by minimum publish date
              paramType: query
            - name: date_max
              type: datetime
              description: Filter posts by maximum publish date
              paramType: query
            - name: search
              type: string
              description: Search for blog posts that match the query
              paramType: query
            - name: page
              type: integer
              description: Page number
              paramType: query
    """
    queryset = Post.objects.published().order_by("-publish_date")
    filter_backends = (filters.DjangoFilterBackend, filters.SearchFilter,)
    search_fields = ('title', 'content',)
    pagination_class = PostPagination
    permission_classes = [IsAdminOrReadOnly]

    def get_serializer_class(self):
            if self.request.method in permissions.SAFE_METHODS:
                return PostOutputSerializer
            else:
                return PostInputSerializer

    def get_filter_class(self):
            if self.request.method in permissions.SAFE_METHODS:
                return PostFilter
            else:
                return None



from __future__ import print_function
import os
import sys
import re
from setuptools import setup, find_packages


def get_version(package):
    """Return package version as listed in `__version__` in `init.py`."""
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep wheel"):
        print("wheel not installed.\nUse `pip install wheel`.\nExiting.")
        sys.exit()
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a {0} -m 'version {0}'".format(get_version("mezzanine_api")))
    print("  git push --tags")
    sys.exit()


setup(
    name='mezzanine-api',
    version=get_version("mezzanine_api"),
    author='George Cushen',
    author_email='mezzanine-users@googlegroups.com',   
    url='http://gcushen.github.io/mezzanine-api',
    description='A RESTful web API for Mezzanine CMS.',
    long_description=open("README.rst", 'rb').read().decode('utf-8'),
    keywords='mezzanine rest restful web api',
    include_package_data=True,
    license='BSD',
    packages=find_packages(),
    install_requires=[
        "Mezzanine >= 4.1.0",
        "django-rest-swagger==0.3.5",
        "djangorestframework==3.3.2",
        "django-filter",
        "django-oauth-toolkit==0.10.0"
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
